'mdMySQLPersistence - by MOD

'no Java representation


#Include Once "mysql\mysql.bi"
#Include Once "md/persistence/mdPersistence.bi"

#Ifndef __MDMYSQLPERSISTENCE_BI__
#Define __MDMYSQLPERSISTENCE_BI__

Type mdMySQLPersistence Extends mdPersistence
	Public:
	Declare Constructor ()
	Declare Constructor (ByRef dbName As String, ByRef host As String, ByRef user As String, ByRef password As String)
	Declare Destructor ()
	
	'CRUD for db
	Declare Virtual Function createDatabase(ByRef dbName As String, ByRef tables As mdMap(String, String)) As mdPersistenceError
	Declare Virtual Function updateDatabase(ByRef dbName As String, ByRef tables As mdMap(String, String)) As mdPersistenceError
	Declare Virtual Function removeDatabase(ByRef dbName As String) As mdPersistenceError
	
	'CRUD for table (persist single objects)
	Declare Virtual Function createObject(ByRef table As String, ByRef newObject As String) As mdPersistenceError
	Declare Virtual Function updateObject(ByRef table As String, ByRef oldObject As String, ByRef newObject As String) As mdPersistenceError
	Declare Virtual Function updateObjectsBy(ByRef table As String, ByRef columnName As String, ByRef value As String, ByRef newObject As String) As mdPersistenceError
	Declare Virtual Function deleteObject(ByRef table As String, ByRef objectString As String) As mdPersistenceError
	Declare Virtual Function deleteAllObjects(ByRef table As String) As mdPersistenceError
	Declare Virtual Function deleteObjectsBy(ByRef table As String, ByRef columnName As String, ByRef value As String) As mdPersistenceError
	Declare Virtual Function getObjects(ByRef table As String, ByRef objectMap As mdMap(String, String)) As mdList(mdMapStringString)
	Declare Virtual Function getObjectsBy(ByRef table As String, ByRef columnName As String, ByRef value As String) As mdList(mdMapStringString)
	
	Declare Virtual Function getLastErrorMessage() As String
	
	Declare Virtual Function dynamicQuery(ByRef queryString As String) As mdList(mdMapStringString)
	
	Declare Virtual Function isConnected() As Boolean
	
	Protected:
	Declare Virtual Function query(ByRef queryString As String) As mdPersistenceError
	
	Declare Virtual Function connect() As mdPersistenceError
	Declare Virtual Function close() As mdPersistenceError
	
	Private:
	As MYSQL Ptr dbPtr
	As String host
	As String user
	As String password
	As Boolean currentlyUsingDB
	
	As mdList(mdMapStringString) _mdMySQLPersistence_resultList_
End Type

MODULE_SCOPE Constructor mdMySQLPersistence()
	
End Constructor

MODULE_SCOPE Constructor mdMySQLPersistence(ByRef dbName As String, ByRef host As String, ByRef user As String, ByRef password As String)
	Base.dbName = dbName
	This.host = host
	This.user = user
	This.password = password
End Constructor

MODULE_SCOPE Destructor mdMySQLPersistence()
	If This.dbPtr <> NULL Then
		If This.isConnected() Then
			This.close()
		EndIf
		'DeAllocate This.dbPtr
		This.dbPtr = NULL
	EndIf
End Destructor

MODULE_SCOPE Function mdMySQLPersistence.createDatabase(ByRef dbName As String, ByRef tables As mdMap(String, String)) As mdPersistenceError
	'create DB
	Base.dbName = dbName
	
	Dim As String queryString = "create database IF NOT EXISTS " & dbName & ";"
	Dim As mdPersistenceError tempError = This.query(queryString)
	If tempError <> mdPersistenceError.NO_ERROR Then
		Return tempError
	EndIf
	
	'create tables
	Dim As mdSet(String) keys
	keys = tables.keySet()
	Dim As String table, column
	ForEach(String, table In keys)
		queryString = "create table IF NOT EXISTS " & table & "("
		Dim As mdMap(String, String) objectMap = mdCollectionsHelper.createMap(tables.get(table))
		Dim As mdSet(String) columns
		columns = objectMap.keySet()
		ForEach(String, column In columns)
			queryString &= column & " varchar(255),"
		NextEach
		
		queryString = Left(queryString, Len(queryString) - 1) 'cut last ","
		queryString &= ");"
		
		This.query(queryString)
		If tempError <> mdPersistenceError.NO_ERROR Then
			Return tempError
		EndIf
	NextEach
	
	Return mdPersistenceError.NO_ERROR
End Function

MODULE_SCOPE Function mdMySQLPersistence.updateDatabase(ByRef dbName As String, ByRef tables As mdMap(String, String)) As mdPersistenceError
	Dim As String queryString = "show tables in " & dbName & ";"
	Dim As mdPersistenceError tempError
	
	Dim As mdSet(String) tablesKeySet
	tablesKeySet = tables.keySet()
	
	Dim As String tempKey
	Dim As mdMap(String, String) tempTables
	ForEach(String, tempKey In tablesKeySet)
		tempKey = tempTables.put(LCase(tempKey), tables.get(tempKey))
	NextEach
	
	tempError = This.query(queryString)
	If tempError <> mdPersistenceError.NO_ERROR Then
		Return tempError
	EndIf
	
	Dim As mdList(mdMapStringString) tempTablesList
	tempTablesList.addAll(_mdMySQLPersistence_resultList_)
	
	Dim As mdMap(String, String) table
	ForEach(mdMapStringString, table In tempTablesList)
		Dim As String tableName = table.get("Tables_in_" & LCase(dbName))
		If Not tempTables.containsKey(tableName) Then
			queryString = "drop table " & tableName & ";"
			tempError = This.query(queryString)
			If tempError <> mdPersistenceError.NO_ERROR Then
				Return tempError
			EndIf
		Else
			queryString = "show columns in " & tableName & " in " & dbName & ";"
			tempError = This.query(queryString)
			If tempError <> mdPersistenceError.NO_ERROR Then
				Return tempError
			EndIf
			
			Dim As mdList(mdMapStringString) tempColumnList
			tempColumnList.addAll(_mdMySQLPersistence_resultList_)
			
			Dim As mdSet(String) oldColumns
			Dim As mdMap(String, String) columnsQueryMap
			ForEach(mdMapStringString, columnsQueryMap In tempColumnList)
				oldColumns.add(columnsQueryMap.get("Field"))
			NextEach
			
			Dim As mdMap(String, String) objectMap = mdCollectionsHelper.createMap(tempTables.get(tableName))
			Dim As mdSet(String) newColumns
			newColumns = objectMap.keySet()
			
			Dim As String tempColumn
			ForEach(String, tempColumn In newColumns)
				If Not oldColumns.contains(tempColumn) Then
					queryString = "alter table " & tableName & " add " & tempColumn & " varchar(255);"
					tempError = This.query(queryString)
					If tempError <> mdPersistenceError.NO_ERROR Then
						Return tempError
					EndIf
				EndIf
			NextEach
			
			ForEach(String, tempColumn In oldColumns)
				If Not newColumns.contains(tempColumn) Then
					queryString = "alter table " & tableName & " drop column " & tempColumn & ";"
					tempError = This.query(queryString)
					If tempError <> mdPersistenceError.NO_ERROR Then
						Return tempError
					EndIf
				EndIf
			NextEach
		EndIf
	NextEach
	
	Return mdPersistenceError.NO_ERROR
End Function

MODULE_SCOPE Function mdMySQLPersistence.removeDatabase(ByRef dbName As String) As mdPersistenceError
	Dim As String queryString = "drop database " & dbName & ";"
	Return This.query(queryString)
End Function

MODULE_SCOPE Function mdMySQLPersistence.createObject(ByRef table As String, ByRef newObject As String) As mdPersistenceError
	Dim As mdMap(String, String) newObjectMap = mdCollectionsHelper.createMap(newObject)
	
	Dim As String queryColumns, queryValues
	Dim As mdSet(String) keys
	keys = newObjectMap.keySet()
	Dim As String column
	ForEach(String, column In keys)
		queryColumns &= column & ","
		queryValues &= "'" & newObjectMap.get(column) & "',"
	NextEach
	
	queryColumns = Left(queryColumns, Len(queryColumns) - 1) 'cut last ","
	queryValues = Left(queryValues, Len(queryValues) - 1) 'cut last ","
	
	Dim As String queryString = "insert into " & table & "(" & queryColumns & ") values (" & queryValues & ");"
	Return This.query(queryString)
End Function

MODULE_SCOPE Function mdMySQLPersistence.updateObject(ByRef table As String, ByRef oldObject As String, ByRef newObject As String) As mdPersistenceError
	Dim As mdMap(String, String) oldObjectMap = mdCollectionsHelper.createMap(oldObject)
	Dim As mdMap(String, String) newObjectMap = mdCollectionsHelper.createMap(newObject)
	
	Dim As String queryWhere, queryValues
	
	Dim As mdSet(String) keys
	keys = oldObjectMap.keySet()
	Dim As String column
	ForEach(String, column In keys)
		queryWhere &= column & "='" & oldObjectMap.get(column) & "' and "
	NextEach
	queryWhere = Left(queryWhere, Len(queryWhere) - 5) 'cut last " and "
	
	keys = newObjectMap.keySet()
	ForEach(String, column In keys)
		queryValues &= column & "='" & newObjectMap.get(column) & "',"
	NextEach
	queryValues = Left(queryValues, Len(queryValues) - 1) 'cut last ","
	
	Dim As String queryString = "update " & table & " set " & queryValues & " where " & queryWhere & ";"
	Return This.query(queryString)
End Function

MODULE_SCOPE Function mdMySQLPersistence.updateObjectsBy(ByRef table As String, ByRef columnName As String, ByRef value As String, ByRef newObject As String) As mdPersistenceError
	Dim As mdMap(String, String) newObjectMap = mdCollectionsHelper.createMap(newObject)
	
	Dim As String queryWhere = columnName & "='" & value & "'"
	Dim As String queryValues
	
	Dim As mdSet(String) keys
	keys = newObjectMap.keySet()
	Dim As String column
	ForEach(String, column In keys)
		queryValues &= column & "='" & newObjectMap.get(column) & "',"
	NextEach
	queryValues = Left(queryValues, Len(queryValues) - 1) 'cut last ","
	
	Dim As String queryString = "update " & table & " set " & queryValues & " where " & queryWhere & ";"
	Return This.query(queryString)
End Function

MODULE_SCOPE Function mdMySQLPersistence.deleteObject(ByRef table As String, ByRef objectString As String) As mdPersistenceError
	Dim As mdMap(String, String) objectMap = mdCollectionsHelper.createMap(objectString)
	
	Dim As String queryValues
	Dim As mdSet(String) keys
	keys = objectMap.keySet()
	Dim As String column
	ForEach(String, column In keys)
		queryValues &= column & "='" & objectMap.get(column) & "' and "
	NextEach
	
	queryValues = Left(queryValues, Len(queryValues) - 5) 'cut last " and "
	
	Dim As String queryString = "delete from " & table & " where " & queryValues & ";"
	Return This.query(queryString)
End Function

MODULE_SCOPE Function mdMySQLPersistence.deleteAllObjects(ByRef table As String) As mdPersistenceError
	Dim As String queryString = "truncate table " & table & ";" '"delete from " & table & ";"
	Return This.query(queryString)
End Function

MODULE_SCOPE Function mdMySQLPersistence.deleteObjectsBy(ByRef table As String, ByRef columnName As String, ByRef value As String) As mdPersistenceError
	Dim As String queryValues = columnName & "='" & value & "'"
	
	Dim As String queryString = "delete from " & table & " where " & queryValues & ";"
	Return This.query(queryString)
End Function

MODULE_SCOPE Function mdMySQLPersistence.getObjects(ByRef table As String, ByRef objectMap As mdMap(String, String)) As mdList(mdMapStringString)
	If This.usingCache = FALSE Then
		Dim As String queryValues
		Dim As mdSet(String) keys
		keys = objectMap.keySet()
		Dim As String column
		ForEach(String, column In keys)
			queryValues &= column & "='" & objectMap.get(column) & "' and "
		NextEach
		
		queryValues = Left(queryValues, Len(queryValues) - 5) 'cut last " and "
		
		Dim As String queryString = "select * from " & table & " where " & queryValues & ";"
		This.query(queryString)
	Else
		
	EndIf
	
	Return _mdMySQLPersistence_resultList_
End Function

MODULE_SCOPE Function mdMySQLPersistence.getObjectsBy(ByRef table As String, ByRef columnName As String, ByRef value As String) As mdList(mdMapStringString)
	If This.usingCache = FALSE Then
		Dim As String queryValues
		If Len(columnName) AndAlso Len(value) Then
			queryValues = " where " & columnName & "='" & value & "'"
		EndIf
		
		Dim As String queryString = "select * from " & table & queryValues & ";"
		This.query(queryString)
	Else
		
	EndIf
	
	Return _mdMySQLPersistence_resultList_
End Function

MODULE_SCOPE Function mdMySQLPersistence.getLastErrorMessage() As String
	Return This.lastErrorMsg
End Function

MODULE_SCOPE Function mdMySQLPersistence.connect() As mdPersistenceError
	If This.isConnected() = FALSE Then
		If This.dbPtr = NULL Then
			This.dbPtr = mysql_init(NULL)
		EndIf
		If mysql_real_connect(This.dbPtr, This.host, This.user, This.password, NULL, MYSQL_PORT, NULL, 0) = NULL Then
			This.lastErrorMsg = "Can't connect to the mysql server on port " & MYSQL_PORT
			mysql_close(This.dbPtr)
			This.dbPtr = NULL
			Return mdPersistenceError.CONNECTION_FAILED
		Else
			Dim As Integer e = mysql_select_db(This.dbPtr, This.dbName)
			If e <> 0 Then
				This.lastErrorMsg = "Can't select the " & This.dbName & " database"
                'mysql_close(This.dbPtr)
				'This.dbPtr = NULL
				'Return mdPersistenceError.CONNECTION_FAILED
				This.currentlyUsingDB = FALSE
			Else
				This.currentlyUsingDB = TRUE
			EndIf
			
			This.currentlyConnected = TRUE
			Return mdPersistenceError.NO_ERROR
		EndIf
	EndIf
End Function

MODULE_SCOPE Function mdMySQLPersistence.isConnected() As Boolean
	Return This.currentlyConnected
End Function

MODULE_SCOPE Function mdMySQLPersistence.close() As mdPersistenceError
	If This.isConnected() Then
		mysql_close(This.dbPtr)
		This.dbPtr = NULL
		This.currentlyConnected = FALSE
	EndIf
	Return mdPersistenceError.NO_ERROR
End Function

MODULE_SCOPE Function mdMySQLPersistence.dynamicQuery(ByRef queryString As String) As mdList(mdMapStringString)
	This.query(queryString)
	Return _mdMySQLPersistence_resultList_
End Function

MODULE_SCOPE Function mdMySQLPersistence.query(ByRef queryString As String) As mdPersistenceError
	If This.isConnected() = FALSE Then
		This.connect()
	EndIf
	If This.isConnected() = FALSE Then
		Return mdPersistenceError.CONNECTION_FAILED
	EndIf
	
	_mdMySQLPersistence_resultList_.clear()
	
	If This.currentlyUsingDB = FALSE Then
		Dim As Integer e = mysql_select_db(This.dbPtr, This.dbName)
		If e <> 0 Then
			This.lastErrorMsg = "Can't select the " & This.dbName & " database"
			This.currentlyUsingDB = FALSE
		Else
			This.currentlyUsingDB = TRUE
		EndIf
	EndIf
	
	Dim As Integer res = mysql_query(This.dbPtr, queryString)
    If res = 0 Then
        Dim As mysql_res Ptr restab = mysql_store_result(This.dbPtr)
        If restab > 0 Then
            Dim As Integer nrow = mysql_num_rows(restab)
			Dim As Integer ncol = mysql_num_fields(restab)
			
			Dim As mdList(String) columnNames
			For i As Integer = 0 To ncol - 1
				Dim As mysql_field Ptr field_ = mysql_fetch_field(restab)
				columnNames.add(*field_->name)
			Next
			
            If nrow > 0 Then
                For i As Integer = 0 To nrow - 1
					Dim As mdMap(String, String) rowMap
                    Dim As mysql_row row = mysql_fetch_row( restab )
                    For j As Integer = 0 To ncol - 1
						Dim As String temp = rowMap.put(columnNames.get(j), *row[j])
                    Next
					_mdMySQLPersistence_resultList_.add(rowMap)
                Next
            EndIf
			
			mysql_free_result( restab )
        EndIf
        
		This.lastErrorMsg = ""
		Return mdPersistenceError.NO_ERROR
    Else
        This.lastErrorMsg = "Error with query '" & queryString & "'"
		Return res
    EndIf
End Function

#EndIf '__MDMYSQLPERSISTENCE_BI__
