'mdPersistence - by MOD

'no Java representation


#Include Once "file.bi"

#Include Once "md/helper/mdCollectionsHelper.bi"
#Include Once "md/helper/mdPersistenceHelper.bi"

#Ifndef __MDPERSISTENCE_BI__
#Define __MDPERSISTENCE_BI__

mdMapDeclare(String, String)
mdListDeclare(String)
mdListDeclare(mdMapStringString)
mdMapDeclare(String, mdListString)

Enum mdPersistenceError
	NOT_IMPLEMENTED = -1
	NO_ERROR = 0
	GENERIC_ERROR
	
	NO_SUCH_DATABASE
	NO_SUCH_TABLE
	
	DATABASE_ALREADY_EXISTS
	
	CONNECTION_FAILED
	
	OBJECT_NOT_UNIQUE
End Enum

Type mdPersistence Extends mdObject
	Public:
	Declare Constructor ()
	Declare Constructor (ByRef dbName As String)
	Declare Destructor ()
	
	'CRUD for db
	Declare Virtual Function createDatabase(ByRef dbName As String, ByRef tables As mdMap(String, String)) As mdPersistenceError
	Declare Virtual Function updateDatabase(ByRef dbName As String, ByRef tables As mdMap(String, String)) As mdPersistenceError
	Declare Virtual Function removeDatabase(ByRef dbName As String) As mdPersistenceError
	
	'CRUD for table (persist single objects)
	Declare Virtual Function createObject(ByRef table As String, ByRef newObject As String) As mdPersistenceError
	Declare Virtual Function updateObject(ByRef table As String, ByRef oldObject As String, ByRef newObject As String) As mdPersistenceError
	Declare Virtual Function updateObjectsBy(ByRef table As String, ByRef columnName As String, ByRef value As String, ByRef newObject As String) As mdPersistenceError
	Declare Virtual Function deleteObject(ByRef table As String, ByRef objectString As String) As mdPersistenceError
	Declare Virtual Function deleteAllObjects(ByRef table As String) As mdPersistenceError
	Declare Virtual Function deleteObjectsBy(ByRef table As String, ByRef columnName As String, ByRef value As String) As mdPersistenceError
	Declare Virtual Function getObjects(ByRef table As String, ByRef objectString As String) As mdList(mdMapStringString)
	Declare Virtual Function getObjectsBy(ByRef table As String, ByRef columnName As String, ByRef value As String) As mdList(mdMapStringString)
	
	Declare Virtual Function getLastErrorMessage() As String
	Declare Virtual Function getDatabaseName() As String
	
	Protected:
	Declare Virtual Function connect() As mdPersistenceError
	Declare Virtual Function isConnected() As Boolean
	Declare Virtual Function close() As mdPersistenceError
	
	Declare Virtual Function query(ByRef queryString As String) As mdPersistenceError
	
	'caching mechanism
	Declare Virtual Function useCache(ByVal use As Boolean) As mdPersistenceError
	Declare Virtual Function createCache(ByRef objectMap As mdList(mdMapStringString)) As mdPersistenceError
	Declare Virtual Function updateCache(ByRef newCache As mdList(mdMapStringString)) As mdPersistenceError
	Declare Virtual Function updateCache(ByRef oldObject As mdMap(String, String), ByRef newObject As mdMap(String, String)) As mdPersistenceError
	Declare Virtual Sub clearCache()
	
	As String dbName
	'As mdMap(String, mdListString) tables
	As Boolean currentlyConnected
	As String lastErrorMsg
	
	As mdList(mdMapStringString) cache
	As Boolean usingCache
End Type

MODULE_SCOPE Constructor mdPersistence()
	
End Constructor

MODULE_SCOPE Constructor mdPersistence(ByRef dbName As String)
	This.dbName = dbName
End Constructor

MODULE_SCOPE Destructor mdPersistence()
	
End Destructor

MODULE_SCOPE Function mdPersistence.createDatabase(ByRef dbName As String, ByRef tables As mdMap(String, String)) As mdPersistenceError
	Return mdPersistenceError.NOT_IMPLEMENTED
End Function

MODULE_SCOPE Function mdPersistence.updateDatabase(ByRef dbName As String, ByRef tables As mdMap(String, String)) As mdPersistenceError
	Return mdPersistenceError.NOT_IMPLEMENTED
End Function

MODULE_SCOPE Function mdPersistence.removeDatabase(ByRef dbName As String) As mdPersistenceError
	Return mdPersistenceError.NOT_IMPLEMENTED
End Function

MODULE_SCOPE Function mdPersistence.createObject(ByRef table As String, ByRef newObject As String) As mdPersistenceError
	Return mdPersistenceError.NOT_IMPLEMENTED
End Function

MODULE_SCOPE Function mdPersistence.updateObject(ByRef table As String, ByRef oldObject As String, ByRef newObject As String) As mdPersistenceError
	Return mdPersistenceError.NOT_IMPLEMENTED
End Function

MODULE_SCOPE Function mdPersistence.updateObjectsBy(ByRef table As String, ByRef columnName As String, ByRef value As String, ByRef newObject As String) As mdPersistenceError
	Return mdPersistenceError.NOT_IMPLEMENTED
End Function

MODULE_SCOPE Function mdPersistence.deleteObject(ByRef table As String, ByRef objectString As String) As mdPersistenceError
	Return mdPersistenceError.NOT_IMPLEMENTED
End Function

MODULE_SCOPE Function mdPersistence.deleteAllObjects(ByRef table As String) As mdPersistenceError
	Return mdPersistenceError.NOT_IMPLEMENTED
End Function

MODULE_SCOPE Function mdPersistence.deleteObjectsBy(ByRef table As String, ByRef columnName As String, ByRef value As String) As mdPersistenceError
	Return mdPersistenceError.NOT_IMPLEMENTED
End Function

MODULE_SCOPE Function mdPersistence.getObjects(ByRef table As String, ByRef objectString As String) As mdList(mdMapStringString)
	Dim As mdList(mdMapStringString) ret
	Return ret
End Function

MODULE_SCOPE Function mdPersistence.getObjectsBy(ByRef table As String, ByRef columnName As String, ByRef value As String) As mdList(mdMapStringString)
	Dim As mdList(mdMapStringString) ret
	Return ret
End Function

MODULE_SCOPE Function mdPersistence.getLastErrorMessage() As String
	Return This.lastErrorMsg
End Function

MODULE_SCOPE Function mdPersistence.getDatabaseName() As String
	Return This.dbName
End Function

MODULE_SCOPE Function mdPersistence.connect() As mdPersistenceError
	Return mdPersistenceError.NOT_IMPLEMENTED
End Function

MODULE_SCOPE Function mdPersistence.isConnected() As Boolean
	Return This.currentlyConnected
End Function

MODULE_SCOPE Function mdPersistence.close() As mdPersistenceError
	Return mdPersistenceError.NOT_IMPLEMENTED
End Function

MODULE_SCOPE Function mdPersistence.query(ByRef queryString As String) As mdPersistenceError
	Return mdPersistenceError.NOT_IMPLEMENTED
End Function

MODULE_SCOPE Function mdPersistence.useCache(ByVal use As Boolean) As mdPersistenceError
	This.usingCache = use
	Return mdPersistenceError.NO_ERROR
End Function

MODULE_SCOPE Function mdPersistence.createCache(ByRef objectMap As mdList(mdMapStringString)) As mdPersistenceError
	Return mdPersistenceError.NOT_IMPLEMENTED
End Function

MODULE_SCOPE Function mdPersistence.updateCache(ByRef newCache As mdList(mdMapStringString)) As mdPersistenceError
	Return mdPersistenceError.NOT_IMPLEMENTED
End Function

MODULE_SCOPE Function mdPersistence.updateCache(ByRef oldObject As mdMap(String, String), ByRef newObject As mdMap(String, String)) As mdPersistenceError
	Return mdPersistenceError.NOT_IMPLEMENTED
End Function

MODULE_SCOPE Sub mdPersistence.clearCache()
	This.cache.clear()
End Sub

#EndIf '__MDPERSISTENCE_BI__
