'mdSQLitePersistence - by MOD

'no Java representation


#Include Once "sqlite3.bi"
#Include Once "md/persistence/mdPersistence.bi"

#Ifndef __MDSQLITEPERSISTENCE_BI__
#Define __MDSQLITEPERSISTENCE_BI__

Dim Shared As mdList(mdMapStringString) _mdSQLitePersistence_resultList_

MODULE_SCOPE Function _mdSQLitePersistence_callback_ Cdecl (ByVal NotUsed As Any Ptr, ByVal argc As Integer, ByVal argv As ZString Ptr Ptr, ByVal colName As ZString Ptr Ptr) As Integer
	Dim As String text
	Dim As mdMap(String, String) newMap
	
	For i As Integer = 0 To argc - 1
		text = "NULL"
		If argv[i] <> 0 Then
			If *argv[i] <> 0 Then 
				text = *argv[i]
			End If
		End If
		
		text = newMap.put(*colName[i], text)
	Next
	
	_mdSQLitePersistence_resultList_.add(newMap)
	Return 0
End Function

Type mdSQLitePersistence Extends mdPersistence
	Public:
	Declare Constructor ()
	Declare Constructor (ByRef dbName As String)
	Declare Destructor ()
	
	'CRUD for db
	Declare Virtual Function createDatabase(ByRef dbName As String, ByRef tables As mdMap(String, String)) As mdPersistenceError
	Declare Virtual Function updateDatabase(ByRef dbName As String, ByRef tables As mdMap(String, String)) As mdPersistenceError
	Declare Virtual Function removeDatabase(ByRef dbName As String) As mdPersistenceError
	
	'CRUD for table (persist single objects)
	Declare Virtual Function createObject(ByRef table As String, ByRef newObject As String) As mdPersistenceError
	Declare Virtual Function updateObject(ByRef table As String, ByRef oldObject As String, ByRef newObject As String) As mdPersistenceError
	Declare Virtual Function updateObjectsBy(ByRef table As String, ByRef columnName As String, ByRef value As String, ByRef newObject As String) As mdPersistenceError
	Declare Virtual Function deleteObject(ByRef table As String, ByRef objectString As String) As mdPersistenceError
	Declare Virtual Function deleteAllObjects(ByRef table As String) As mdPersistenceError
	Declare Virtual Function deleteObjectsBy(ByRef table As String, ByRef columnName As String, ByRef value As String) As mdPersistenceError
	Declare Virtual Function getObjects(ByRef table As String, ByRef objectMap As mdMap(String, String)) As mdList(mdMapStringString)
	Declare Virtual Function getObjectsBy(ByRef table As String, ByRef columnName As String, ByRef value As String) As mdList(mdMapStringString)
	
	Declare Virtual Function getLastErrorMessage() As String
	
	Declare Virtual Function dynamicQuery(ByRef queryString As String) As mdList(mdMapStringString)
	
	Declare Virtual Function isConnected() As Boolean
	
	Protected:
	Declare Virtual Function query(ByRef queryString As String) As mdPersistenceError
	
	Declare Virtual Function connect() As mdPersistenceError
	Declare Virtual Function close() As mdPersistenceError
	
	Private:
	As sqlite3 Ptr dbPtr
End Type

MODULE_SCOPE Constructor mdSQLitePersistence()
	
End Constructor

MODULE_SCOPE Constructor mdSQLitePersistence(ByRef dbName As String)
	Base.dbName = dbName
End Constructor

MODULE_SCOPE Destructor mdSQLitePersistence()
	If This.dbPtr <> NULL Then
		If This.isConnected() Then
			This.close()
		EndIf
		'DeAllocate This.dbPtr - already done by sqlite
		This.dbPtr = NULL
	EndIf
End Destructor

MODULE_SCOPE Function mdSQLitePersistence.createDatabase(ByRef dbName As String, ByRef tables As mdMap(String, String)) As mdPersistenceError
	'create DB
	Base.dbName = dbName
	If FileExists(Base.dbName) Then
		Return mdPersistenceError.DATABASE_ALREADY_EXISTS
	EndIf
	Dim As String queryString' = "create database " & dbName & ";"
	Dim As mdPersistenceError tempError' = This.query(queryString)
	'If tempError <> mdPersistenceError.NO_ERROR Then
	'	Return tempError
	'EndIf
	
	'create tables
	Dim As mdSet(String) keys
	keys = tables.keySet()
	Dim As String table, column
	ForEach(String, table In keys)
		queryString = "create table " & table & "("
		Dim As mdMap(String, String) objectMap = mdCollectionsHelper.createMap(tables.get(table))
		Dim As mdSet(String) columns
		columns = objectMap.keySet()
		ForEach(String, column In columns)
			queryString &= column & " varchar(255),"
		NextEach
		
		queryString = Left(queryString, Len(queryString) - 1) 'cut last ","
		queryString &= ");"
		
		This.query(queryString)
		If tempError <> mdPersistenceError.NO_ERROR Then
			Return tempError
		EndIf
	NextEach
	
	Return mdPersistenceError.NO_ERROR
End Function

MODULE_SCOPE Function mdSQLitePersistence.updateDatabase(ByRef dbName As String, ByRef tables As mdMap(String, String)) As mdPersistenceError
	Dim As String queryString = "select * from sqlite_master where type='table';"
	Dim As mdPersistenceError tempError
	
	tempError = This.query(queryString)
	If tempError <> mdPersistenceError.NO_ERROR Then
		Return tempError
	EndIf
	
	Dim As mdList(mdMapStringString) tempTablesList
	tempTablesList.addAll(_mdSQLitePersistence_resultList_)
	
	Dim As mdMap(String, String) table
	ForEach(mdMapStringString, table In tempTablesList)
		Dim As String tableName = table.get("name")
		If Not tables.containsKey(tableName) Then
			queryString = "drop table " & tableName & ";"
			tempError = This.query(queryString)
			If tempError <> mdPersistenceError.NO_ERROR Then
				Return tempError
			EndIf
		Else
			Dim As mdString tableSql = table.get("sql")
			Dim As mdList(String) oldColumns
			tableSql = tableSql.replace("CREATE TABLE " & tableName & "(", "")
			tableSql = Left(tableSql, tableSql.length() - 1)
			Dim As String tableSqlParts()
			tableSql.split(tableSqlParts(), ",")
			For i As Integer = LBound(tableSqlParts) To UBound(tableSqlParts)
				oldColumns.add(Left(tableSqlParts(i), InStr(tableSqlParts(i), " ") - 1))
			Next
			
			Dim As mdMap(String, String) objectMap = mdCollectionsHelper.createMap(tables.get(tableName))
			Dim As mdSet(String) newColumns
			newColumns = objectMap.keySet()
			
			Dim As String tempColumn
			ForEach(String, tempColumn In newColumns)
				If Not oldColumns.contains(tempColumn) Then
					queryString = "alter table " & tableName & " add " & tempColumn & " varchar(255);"
					tempError = This.query(queryString)
					If tempError <> mdPersistenceError.NO_ERROR Then
						Return tempError
					EndIf
				EndIf
			NextEach
			
			'not possible in sqlite
			'ForEach(String, tempColumn In oldColumns)
			'	If Not newColumns.contains(tempColumn) Then
			'		queryString = "alter table " & tableName & " drop column " & tempColumn & ";"
			'		tempError = This.query(queryString)
			'		If tempError <> mdPersistenceError.NO_ERROR Then
			'			Return tempError
			'		EndIf
			'	EndIf
			'NextEach
		EndIf
	NextEach
	
	Return mdPersistenceError.NO_ERROR
End Function

MODULE_SCOPE Function mdSQLitePersistence.removeDatabase(ByRef dbName As String) As mdPersistenceError
	'Dim As String queryString = "drop database " & dbName & ";"
	'This.query(queryString)
	Dim As mdPersistenceError tempError = This.close()
	
	If tempError = mdPersistenceError.NO_ERROR Then
		Kill dbName
	EndIf
	
	Return tempError
End Function

MODULE_SCOPE Function mdSQLitePersistence.createObject(ByRef table As String, ByRef newObject As String) As mdPersistenceError
	Dim As mdMap(String, String) newObjectMap = mdCollectionsHelper.createMap(newObject)
	
	Dim As String queryColumns, queryValues
	Dim As mdSet(String) keys
	keys = newObjectMap.keySet()
	Dim As String column
	ForEach(String, column In keys)
		queryColumns &= column & ","
		queryValues &= "'" & newObjectMap.get(column) & "',"
	NextEach
	
	queryColumns = Left(queryColumns, Len(queryColumns) - 1) 'cut last ","
	queryValues = Left(queryValues, Len(queryValues) - 1) 'cut last ","
	
	Dim As String queryString = "insert into " & table & "(" & queryColumns & ") values (" & queryValues & ");"
	Return This.query(queryString)
End Function

MODULE_SCOPE Function mdSQLitePersistence.updateObject(ByRef table As String, ByRef oldObject As String, ByRef newObject As String) As mdPersistenceError
	Dim As mdMap(String, String) oldObjectMap = mdCollectionsHelper.createMap(oldObject)
	Dim As mdMap(String, String) newObjectMap = mdCollectionsHelper.createMap(newObject)
	
	Dim As String queryWhere, queryValues
	
	Dim As mdSet(String) keys
	keys = oldObjectMap.keySet()
	Dim As String column
	ForEach(String, column In keys)
		queryWhere &= column & "='" & oldObjectMap.get(column) & "' and "
	NextEach
	queryWhere = Left(queryWhere, Len(queryWhere) - 5) 'cut last " and "
	
	keys = newObjectMap.keySet()
	ForEach(String, column In keys)
		queryValues &= column & "='" & newObjectMap.get(column) & "',"
	NextEach
	queryValues = Left(queryValues, Len(queryValues) - 1) 'cut last ","
	
	Dim As String queryString = "update " & table & " set " & queryValues & " where " & queryWhere & ";"
	Return This.query(queryString)
End Function

MODULE_SCOPE Function mdSQLitePersistence.updateObjectsBy(ByRef table As String, ByRef columnName As String, ByRef value As String, ByRef newObject As String) As mdPersistenceError
	Dim As mdMap(String, String) newObjectMap = mdCollectionsHelper.createMap(newObject)
	
	Dim As String queryWhere = columnName & "='" & value & "'"
	Dim As String queryValues
	
	Dim As mdSet(String) keys
	keys = newObjectMap.keySet()
	Dim As String column
	ForEach(String, column In keys)
		queryValues &= column & "='" & newObjectMap.get(column) & "',"
	NextEach
	queryValues = Left(queryValues, Len(queryValues) - 1) 'cut last ","
	
	Dim As String queryString = "update " & table & " set " & queryValues & " where " & queryWhere & ";"
	Return This.query(queryString)
End Function

MODULE_SCOPE Function mdSQLitePersistence.deleteObject(ByRef table As String, ByRef objectString As String) As mdPersistenceError
	Dim As mdMap(String, String) objectMap = mdCollectionsHelper.createMap(objectString)
	
	Dim As String queryValues
	Dim As mdSet(String) keys
	keys = objectMap.keySet()
	Dim As String column
	ForEach(String, column In keys)
		queryValues &= column & "='" & objectMap.get(column) & "' and "
	NextEach
	
	queryValues = Left(queryValues, Len(queryValues) - 5) 'cut last " and "
	
	Dim As String queryString = "delete from " & table & " where " & queryValues & ";"
	Return This.query(queryString)
End Function

MODULE_SCOPE Function mdSQLitePersistence.deleteAllObjects(ByRef table As String) As mdPersistenceError
	Dim As String queryString = "delete from " & table & ";"'"truncate table " & table & ";"
	Return This.query(queryString)
End Function

MODULE_SCOPE Function mdSQLitePersistence.deleteObjectsBy(ByRef table As String, ByRef columnName As String, ByRef value As String) As mdPersistenceError
	Dim As String queryValues = columnName & "='" & value & "'"
	
	Dim As String queryString = "delete from " & table & " where " & queryValues & ";"
	Return This.query(queryString)
End Function

MODULE_SCOPE Function mdSQLitePersistence.getObjects(ByRef table As String, ByRef objectMap As mdMap(String, String)) As mdList(mdMapStringString)
	If This.usingCache = FALSE Then
		Dim As String queryValues
		Dim As mdSet(String) keys
		keys = objectMap.keySet()
		Dim As String column
		ForEach(String, column In keys)
			queryValues &= column & "='" & objectMap.get(column) & "' and "
		NextEach
		
		queryValues = Left(queryValues, Len(queryValues) - 5) 'cut last " and "
		
		Dim As String queryString = "select * from " & table & " where " & queryValues & ";"
		This.query(queryString)
	Else
		
	EndIf
	
	Return _mdSQLitePersistence_resultList_
End Function

MODULE_SCOPE Function mdSQLitePersistence.getObjectsBy(ByRef table As String, ByRef columnName As String, ByRef value As String) As mdList(mdMapStringString)
	If This.usingCache = FALSE Then
		Dim As String queryValues
		If Len(columnName) AndAlso Len(value) Then
			queryValues = " where " & columnName & "='" & value & "'"
		EndIf
		
		Dim As String queryString = "select * from " & table & queryValues & ";"
		This.query(queryString)
	Else
		
	EndIf
	
	Return _mdSQLitePersistence_resultList_
End Function

MODULE_SCOPE Function mdSQLitePersistence.getLastErrorMessage() As String
	Return This.lastErrorMsg
End Function

MODULE_SCOPE Function mdSQLitePersistence.connect() As mdPersistenceError
	If Len(This.dbName) = 0 Then
		Return mdPersistenceError.NO_SUCH_DATABASE
	EndIf
	
	If This.isConnected() = FALSE Then
		Dim As Integer e = sqlite3_open(This.dbName, @This.dbPtr)
		If e = mdPersistenceError.NO_ERROR Then
			This.currentlyConnected = TRUE
		EndIf
		Return e
	EndIf
	Return mdPersistenceError.CONNECTION_FAILED
End Function

MODULE_SCOPE Function mdSQLitePersistence.isConnected() As Boolean
	Return This.currentlyConnected
End Function

MODULE_SCOPE Function mdSQLitePersistence.close() As mdPersistenceError
	If This.isConnected() Then
		sqlite3_close(This.dbPtr)
		This.currentlyConnected = FALSE
	EndIf
	Return mdPersistenceError.NO_ERROR
End Function

MODULE_SCOPE Function mdSQLitePersistence.dynamicQuery(ByRef queryString As String) As mdList(mdMapStringString)
	This.query(queryString)
	Return _mdSQLitePersistence_resultList_
End Function

MODULE_SCOPE Function mdSQLitePersistence.query(ByRef queryString As String) As mdPersistenceError
	If This.isConnected() = FALSE Then
		This.connect()
	EndIf
	If This.isConnected() = FALSE Then
		Return mdPersistenceError.CONNECTION_FAILED
	EndIf
	
	Dim As ZString Ptr errMsg
	_mdSQLitePersistence_resultList_.clear()
	Dim As Integer errNr = sqlite3_exec(This.dbPtr, queryString, @_mdSQLitePersistence_callback_, 0, @errMsg)
	If errNr <> SQLITE_OK Then
		This.lastErrorMsg = *errMsg
		Return errNr
	Else
		This.lastErrorMsg = ""
		Return mdPersistenceError.NO_ERROR
	EndIf
End Function

#EndIf '__MDSQLITEPERSISTENCE_BI__
