'mdTypesPersistence - by MOD

'no Java representation


#Include Once "md/persistence/mdPersistence.bi"

#Ifndef __MDTYPESPERSISTENCE_BI__
#Define __MDTYPESPERSISTENCE_BI__

Type mdTypesPersistence Extends mdPersistence
	Public:
	Declare Constructor ()
	Declare Constructor (ByRef dbName As String)
	Declare Destructor ()
	
	'CRUD for db
	Declare Virtual Function createDatabase(ByRef dbName As String, ByRef tables As mdMap(String, String)) As mdPersistenceError
	Declare Virtual Function updateDatabase(ByRef dbName As String, ByRef tables As mdMap(String, String)) As mdPersistenceError
	Declare Virtual Function removeDatabase(ByRef dbName As String) As mdPersistenceError
	
	'CRUD for table (persist single objects)
	Declare Virtual Function createObject(ByRef table As String, ByRef newObject As String) As mdPersistenceError
	Declare Virtual Function updateObject(ByRef table As String, ByRef oldObject As String, ByRef newObject As String) As mdPersistenceError
	Declare Virtual Function updateObjectsBy(ByRef table As String, ByRef columnName As String, ByRef value As String, ByRef newObject As String) As mdPersistenceError
	Declare Virtual Function deleteObject(ByRef table As String, ByRef objectString As String) As mdPersistenceError
	Declare Virtual Function deleteAllObjects(ByRef table As String) As mdPersistenceError
	Declare Virtual Function deleteObjectsBy(ByRef table As String, ByRef columnName As String, ByRef value As String) As mdPersistenceError
	Declare Virtual Function getObjects(ByRef table As String, ByRef objectString As String) As mdList(mdMapStringString)
	Declare Virtual Function getObjectsBy(ByRef table As String, ByRef columnName As String, ByRef value As String) As mdList(mdMapStringString)
	
	Declare Virtual Function getLastErrorMessage() As String
	
	Protected:
	Declare Virtual Function connect() As mdPersistenceError
	Declare Virtual Function isConnected() As Boolean
	Declare Virtual Function close() As mdPersistenceError
	
	Declare Virtual Function query(ByRef queryString As String) As mdPersistenceError
	
	Private:
	As Integer fileNr
	As mdMap(String, mdListString) tables
End Type

MODULE_SCOPE Constructor mdTypesPersistence()
	
End Constructor

MODULE_SCOPE Constructor mdTypesPersistence(ByRef dbName As String)
	Base.dbName = dbName
	This.connect()
End Constructor

MODULE_SCOPE Destructor mdTypesPersistence()
	If FileExists(This.dbName) Then
		This.close()
	EndIf
End Destructor

MODULE_SCOPE Function mdTypesPersistence.createDatabase(ByRef dbName As String, ByRef tables As mdMap(String, String)) As mdPersistenceError
	Base.dbName = dbName
	
	Dim As mdPersistenceError tempError = This.connect()
	
	Dim As mdList(String) emptyList
	
	Dim As mdSet(String) keys
	keys = tables.keySet()
	Dim As String table, temp
	ForEach(String, table In keys)
		emptyList = This.tables.put(table, emptyList)
	NextEach
	
	Return tempError
End Function

MODULE_SCOPE Function mdTypesPersistence.updateDatabase(ByRef dbName As String, ByRef tables As mdMap(String, String)) As mdPersistenceError
	Return mdPersistenceError.NOT_IMPLEMENTED
End Function

MODULE_SCOPE Function mdTypesPersistence.removeDatabase(ByRef dbName As String) As mdPersistenceError
	Dim As mdPersistenceError tempError = This.close()
	
	If tempError = mdPersistenceError.NO_ERROR Then
		Kill dbName
	EndIf
	
	Return tempError
End Function

MODULE_SCOPE Function mdTypesPersistence.createObject(ByRef table As String, ByRef newObject As String) As mdPersistenceError
	If Not This.tables.containsKey(table) Then
		Return mdPersistenceError.NO_SUCH_TABLE
	EndIf
	
	Dim As mdList(String) currTableList = This.tables.get(table)
	currTableList.add(newObject)
	currTableList = This.tables.put(table, currTableList)
	
	Return mdPersistenceError.NO_ERROR
End Function

MODULE_SCOPE Function mdTypesPersistence.updateObject(ByRef table As String, ByRef oldObject As String, ByRef newObject As String) As mdPersistenceError
	If Not This.tables.containsKey(table) Then
		Return mdPersistenceError.NO_SUCH_TABLE
	EndIf
	
	Dim As mdList(String) currTableList = This.tables.get(table)
	For i As Integer = 0 To currTableList.size() - 1
		If currTableList.get(i) = oldObject Then
			Dim As String temp = currTableList.set(i, newObject)
		EndIf
	Next
	
	currTableList = This.tables.put(table, currTableList)
	
	Return mdPersistenceError.NO_ERROR
End Function

MODULE_SCOPE Function mdTypesPersistence.updateObjectsBy(ByRef table As String, ByRef columnName As String, ByRef value As String, ByRef newObject As String) As mdPersistenceError
	If Not This.tables.containsKey(table) Then
		Return mdPersistenceError.NO_SUCH_TABLE
	EndIf
	
	Dim As mdList(String) currTableList = This.tables.get(table)
	For i As Integer = 0 To currTableList.size() - 1
		Dim As mdMap(String, String) tempObjectMap = mdCollectionsHelper.createMap(currTableList.get(i))
		If tempObjectMap.get(columnName) = value Then
			Dim As String temp = currTableList.set(i, newObject)
		EndIf
	Next
	
	currTableList = This.tables.put(table, currTableList)
	
	Return mdPersistenceError.NO_ERROR
End Function

MODULE_SCOPE Function mdTypesPersistence.deleteObject(ByRef table As String, ByRef objectString As String) As mdPersistenceError
	If Not This.tables.containsKey(table) Then
		Return mdPersistenceError.NO_SUCH_TABLE
	EndIf
	
	Dim As mdList(String) currTableList = This.tables.get(table)
	currTableList.remove(objectString)
	
	Dim As mdList(String) prevTableList = This.tables.put(table, currTableList)
	
	Return mdPersistenceError.NO_ERROR
End Function

MODULE_SCOPE Function mdTypesPersistence.deleteAllObjects(ByRef table As String) As mdPersistenceError
	If Not This.tables.containsKey(table) Then
		Return mdPersistenceError.NO_SUCH_TABLE
	EndIf
	
	Dim As mdList(String) currTableList = This.tables.get(table)
	currTableList.clear()
	
	currTableList = This.tables.put(table, currTableList)
	
	Return mdPersistenceError.NO_ERROR
End Function

MODULE_SCOPE Function mdTypesPersistence.deleteObjectsBy(ByRef table As String, ByRef columnName As String, ByRef value As String) As mdPersistenceError
	If Not This.tables.containsKey(table) Then
		Return mdPersistenceError.NO_SUCH_TABLE
	EndIf
	
	Dim As mdList(String) currTableList = This.tables.get(table)
	Dim As mdList(String) tempTableList = currTableList
	For i As Integer = 0 To tempTableList.size() - 1
		Dim As mdMap(String, String) tempObjectMap = mdCollectionsHelper.createMap(tempTableList.get(i))
		If tempObjectMap.get(columnName) = value Then
			Dim As String temp = currTableList.remove(i)
		EndIf
	Next
	
	currTableList = This.tables.put(table, currTableList)
	
	Return mdPersistenceError.NO_ERROR
End Function

MODULE_SCOPE Function mdTypesPersistence.getObjects(ByRef table As String, ByRef objectString As String) As mdList(mdMapStringString)
	Dim As mdList(mdMapStringString) ret
	
	Dim As mdList(String) currTableList = This.tables.get(table)
	For i As Integer = 0 To currTableList.size() - 1
		If currTableList.get(i) = objectString Then
			ret.add(mdCollectionsHelper.createMap(currTableList.get(i)))
		EndIf
	Next
	
	Return ret
End Function

MODULE_SCOPE Function mdTypesPersistence.getObjectsBy(ByRef table As String, ByRef columnName As String, ByRef value As String) As mdList(mdMapStringString)
	Dim As mdList(mdMapStringString) ret
	
	Dim As mdList(String) currTableList = This.tables.get(table)
	For i As Integer = 0 To currTableList.size() - 1
		Dim As mdMap(String, String) tempObjectMap = mdCollectionsHelper.createMap(currTableList.get(i))
		If tempObjectMap.get(columnName) = value Then
			ret.add(mdCollectionsHelper.createMap(currTableList.get(i)))
		EndIf
	Next
	
	Return ret
End Function

MODULE_SCOPE Function mdTypesPersistence.getLastErrorMessage() As String
	Return This.lastErrorMsg
End Function

MODULE_SCOPE Function mdTypesPersistence.connect() As mdPersistenceError
	If FileExists(Base.dbName) = FALSE Then
		This.fileNr = FreeFile
		Open Base.dbName For Output As #This.fileNr
		This.currentlyConnected = TRUE
		.Close #This.fileNr
		This.currentlyConnected = FALSE
	EndIf
	
	If This.currentlyConnected = TRUE Then
		.Close #This.fileNr
		This.currentlyConnected = FALSE
	EndIf
	
	This.fileNr = FreeFile
	Open Base.dbName For Input As #This.fileNr
	This.currentlyConnected = TRUE
	
	Dim As String temp, contentString
	While Not Eof(This.fileNr)
		Line Input #This.fileNr, temp
		contentString &= temp
	Wend
	
	Dim As mdList(String) tempList
	Dim As mdMap(String, String) contentSimpleMap = mdCollectionsHelper.createMap(contentString)
	Dim As mdSet(String) contentTables = contentSimpleMap.keySet()
	ForEach(String, temp In contentTables)
		tempList = This.tables.put(temp, mdCollectionsHelper.createList(contentSimpleMap.get(temp)))
	NextEach
	
	This.currentlyConnected = TRUE
	
	Return mdPersistenceError.NO_ERROR
End Function

MODULE_SCOPE Function mdTypesPersistence.isConnected() As Boolean
	Return This.currentlyConnected
End Function

MODULE_SCOPE Function mdTypesPersistence.close() As mdPersistenceError
	Dim As String tempContent = This.tables.toString()
	
	If This.currentlyConnected = TRUE Then
		.Close #This.fileNr
	EndIf
	
	This.fileNr = FreeFile
	Open Base.dbName For Output As #This.fileNr 
	This.currentlyConnected = TRUE
		Put #This.fileNr, 0, tempContent
	.Close #This.fileNr 
	This.currentlyConnected = FALSE
	
	Return mdPersistenceError.NO_ERROR
End Function

MODULE_SCOPE Function mdTypesPersistence.query(ByRef queryString As String) As mdPersistenceError
	Return mdPersistenceError.NOT_IMPLEMENTED
End Function

#EndIf '__MDTYPESPERSISTENCE_BI__
