'mdXMLFormatter - by MOD

'http://docs.oracle.com/javase/6/docs/api/java/util/logging/XMLFormatter.html


#Include Once "md/util/logging/mdFormatter.bi"


#Ifndef __MDXMLFORMATTER_BI__
#Define __MDXMLFORMATTER_BI__

Type mdXMLFormatter Extends mdFormatter
	Public:
	Declare Virtual /'Abstract'/ Function format(ByRef record As mdLogRecord) As String
	Declare Virtual Function getHead(ByVal h As mdHandlerAlias Ptr) As String
	Declare Virtual Function getTail(ByVal h As mdHandlerAlias Ptr) As String
	
	'Protected:
	Declare Constructor ()
	
	Private:
	Declare Sub escape(ByRef sb As String, ByRef text As String)
End Type

MODULE_SCOPE Constructor mdXMLFormatter()
	
End Constructor

MODULE_SCOPE Function mdXMLFormatter.format(ByRef record As mdLogRecord) As String
	Dim As String ret
	ret &= "  <record>" & NEWLINE
	
	Dim As Double millis = record.getMillisDouble()
	ret &= "    <date>" & ..Format(millis, "mm-dd-yyyy hh:mm:ss ") & "</date>" & NEWLINE
	ret &= "    <millis>" & millis & "</millis>" & NEWLINE
	ret &= "    <sequence>" & record.getSequenceNumber & "</sequence>" & NEWLINE
	
	Dim As String loggerName = record.getLoggerName()
	If loggerName <> "" Then
		ret &= "    <logger>" & loggerName & "</logger>" & NEWLINE
	EndIf
	
	ret &= "    <level>" & record.getLevel().toString() & "</level>" & NEWLINE
	
	If record.getSourceClassName() <> "" Then
		ret &= "    <class>" & record.getSourceClassName() & "</class>" & NEWLINE
	EndIf
	If record.getSourceMethodName() <> "" Then
		ret &= "    <method>" & record.getSourceMethodName() & "</method>" & NEWLINE
	EndIf
	
	ret &= "    <thread>" & record.getThreadID() & "</thread>" & NEWLINE
	
	If record.getMessage() <> "" Then
		Dim As String message = This.formatMessage(record)
		ret &= "    <message>"
		This.escape(ret, message)
		ret &= "</message>" & NEWLINE
	EndIf
	
	Dim As mdResourceBundle bundle = record.getResourceBundle()
	If bundle.getString(record.getMessage()) <> "" Then
		ret &= "    <key>"
		This.escape(ret, record.getMessage())
		ret &= "</key>" & NEWLINE
		ret &= "    <catalog>"
		This.escape(ret, record.getResourceBundleName())
		ret &= "</catalog>" & NEWLINE
	EndIf
	
	Dim As String parameters()
	record.getParameters(parameters())
	If @parameters(0) <> NULL Then
		For i As Integer = LBound(parameters) To UBound(parameters)
			ret &= "    <param>"
			This.escape(ret, parameters(i))
			ret &= "</param>" & NEWLINE
		Next
	EndIf
	
	Dim As mdThrowable th = record.getThrown()
	If Trim(th.getStackTrace(), NEWLINE) <> "" Then
		ret &= "    <exception>" & NEWLINE
		ret &= "      <message>"
		This.escape(ret, th.toString())
		ret &= "</message>" & NEWLINE
		
		Dim As mdStackTraceElement trace()
		th.getStackTrace(trace())
		For i As Integer = LBound(trace) To UBound(trace)
			ret &= "      <frame>" & NEWLINE
			ret &= "        <class>"
			This.escape(ret, trace(i).getClassName())
			ret &= "</class>" & NEWLINE
			ret &= "        <method>"
			This.escape(ret, trace(i).getMethodName())
			ret &= "</method>" & NEWLINE
			If trace(i).getLineNumber() > 0 Then
				ret &= "        <line>" & trace(i).getLineNumber() & "</line>" & NEWLINE
			EndIf
			ret &= "      </frame>" & NEWLINE
		Next
		ret &= "    </exception>" & NEWLINE
	EndIf
	
	ret &= "  </record>" & NEWLINE
	Return ret
End Function

MODULE_SCOPE Function mdXMLFormatter.getHead(ByVal h As mdHandlerAlias Ptr) As String
	Dim As String ret = !"<?xml version=\"1.0\""
	Dim As String encoding_
	If h <> NULL Then
		encoding_ = h->getEncoding()
	Else
		encoding_ = "ASCII"
	EndIf
	ret &= !" encoding=\"" & encoding_ & !"\""
	ret &= !" standalone=\"no\"?>" + NEWLINE
	ret &= !"<!DOCTYPE log SYSTEM \"logger.dtd\">" + NEWLINE
	ret &= "<log>" + NEWLINE
	Return ret
End Function

MODULE_SCOPE Function mdXMLFormatter.getTail(ByVal h As mdHandlerAlias Ptr) As String
	Dim As String ret = "</log>" + NEWLINE
	Return ret
End Function

MODULE_SCOPE Sub mdXMLFormatter.escape(ByRef sb As String, ByRef text As String)
	For i As Integer = 0 To Len(text) - 1
		Select Case text[i]
			Case 60 '<
				sb &= "&lt;"
			Case 62 '>
				sb &= "&gt;"
			Case 38 '&
				sb &= "&amp;"
			Case Else
				sb &= Chr(text[i])
		End Select
	Next
End Sub

#EndIf '__MDXMLFORMATTER_BI__
