#Include Once "md/persistence/mdMySQLPersistence.bi"

'Persisted classes
Type MyFoo
	Declare Constructor ()
	Declare Constructor (ByRef newObject As mdMap(String, String))
	
	Declare Operator Cast() As String
	
	As Integer id
	As String description
End Type

Constructor MyFoo()
	
End Constructor

Constructor MyFoo(ByRef newObject As mdMap(String, String))
	This.id = ValInt(newObject.get("id"))
	This.description = mdCollectionsHelper.decode(newObject.get("description"))
End Constructor

Operator MyFoo.Cast() As String
	'Return all relevant fields and values as map
	Return "{id=" & This.id & ", description=" & mdCollectionsHelper.encode(This.description) & "}"
End Operator

Operator = (ByRef lhs As MyFoo, ByRef rhs As MyFoo) As Boolean
	Return Str(lhs) = Str(rhs)
End Operator

Type MyBar
	Declare Constructor ()
	Declare Constructor (ByRef newObject As mdMap(String, String))
	
	Declare Operator Cast() As String
	
	As Integer id
	As Double value
	As Boolean isImportant
End Type

Constructor MyBar()
	
End Constructor

Constructor MyBar(ByRef newObject As mdMap(String, String))
	This.id = ValInt(newObject.get("id"))
	This.value = Val(newObject.get("value"))
	This.isImportant = ValInt(newObject.get("isImportant"))
End Constructor

Operator MyBar.Cast() As String
	'Return all relevant fields and values as map
	Return "{id=" & This.id & ", value=" & This.value & ", isImportant=" & This.isImportant & "}"
End Operator

Operator = (ByRef lhs As MyBar, ByRef rhs As MyBar) As Boolean
	Return Str(lhs) = Str(rhs)
End Operator

#Define PERSISTENCE_CLASS mdMySQLPersistence
#Define DB_NAME "MyApplicationDB_mdMySQLPersistence"
#Define DB_HOST "localhost"
#Define DB_USER "root"
#Define DB_PWD ""

Sub create_and_use_new_db()
	Dim As MyFoo tempFoo
	Dim As MyBar tempBar
	
	'Prepare table information
	Dim As String temp
	Dim As mdMap(String, String) tables
	temp = tables.put("MyFoo", tempFoo)
	temp = tables.put("MyBar", tempBar)
	
	'Create database
	Dim As PERSISTENCE_CLASS sql = Type<PERSISTENCE_CLASS>(DB_NAME, DB_HOST, DB_USER, DB_PWD)
	sql.createDatabase(DB_NAME, tables)
	
	'Create sample data and fill tables
	tempFoo.id = 1 : tempFoo.description = "1" : sql.createObject("MyFoo", tempFoo)
	tempFoo.id = 2 : tempFoo.description = "2" : sql.createObject("MyFoo", tempFoo)
	tempFoo.id = 3 : tempFoo.description = "3" : sql.createObject("MyFoo", tempFoo)
	tempFoo.id = 4 : tempFoo.description = "4" : sql.createObject("MyFoo", tempFoo)
	tempFoo.id = 5 : tempFoo.description = "5" : sql.createObject("MyFoo", tempFoo)
	tempFoo.id = 6 : tempFoo.description = "6" : sql.createObject("MyFoo", tempFoo)
	tempFoo.id = 7 : tempFoo.description = "7" : sql.createObject("MyFoo", tempFoo)
	
	tempBar.id = 1 : tempBar.value = 1.1 : tempBar.isImportant = TRUE : sql.createObject("MyBar", tempBar)
	tempBar.id = 2 : tempBar.value = 2.2 : tempBar.isImportant = TRUE : sql.createObject("MyBar", tempBar)
	tempBar.id = 3 : tempBar.value = 3.3 : tempBar.isImportant = TRUE : sql.createObject("MyBar", tempBar)
	tempBar.id = 4 : tempBar.value = 4.4 : tempBar.isImportant = TRUE : sql.createObject("MyBar", tempBar)
	tempBar.id = 5 : tempBar.value = 5.5 : tempBar.isImportant = TRUE : sql.createObject("MyBar", tempBar)
	
	'Get sample objects
	Dim As mdList(mdMapStringString) resultList
	
	resultList = sql.getObjectsBy("MyFoo", "id", "2")
	Assert(resultList = "[{id=2, description=2}]")
	Assert(sql.getLastErrorMessage() = "")
	
	resultList = sql.getObjectsBy("MyFoo", "id", "5")
	Assert(resultList = "[{id=5, description=5}]")
	Assert(sql.getLastErrorMessage() = "")
	
	resultList = sql.getObjectsBy("MyBar", "value", "3.3")
	Assert(resultList = "[{id=3, value=3.3, isImportant=-1}]")
	Assert(sql.getLastErrorMessage() = "")
	
	'Update some objects
	Dim As MyFoo newFoo : newFoo.id = 2 : newFoo.description = "22"
	sql.updateObjectsBy("MyFoo", "id", "2", newFoo)
	Assert(sql.getLastErrorMessage() = "")
	resultList = sql.getObjectsBy("MyFoo", "id", "2")
	Assert(resultList = "[{id=2, description=22}]")
	Assert(sql.getLastErrorMessage() = "")
	
	Dim As MyFoo oldFoo : oldFoo.id = 3 : oldFoo.description = "3"
	newFoo.id = 3 : newFoo.description = "33"
	sql.updateObject("MyFoo", oldFoo, newFoo)
	Assert(sql.getLastErrorMessage() = "")
	resultList = sql.getObjectsBy("MyFoo", "id", "3")
	Assert(resultList = "[{id=3, description=33}]")
	Assert(sql.getLastErrorMessage() = "")
	
	'Remove some objects
	Dim As MyBar barToRemove : barToRemove.id = 2 : barToRemove.value = 2.2 : barToRemove.isImportant = TRUE
	resultList = sql.getObjectsBy("MyBar", "", "")
	Assert(resultList.size() = 5)
	
	sql.deleteObject("MyBar", barToRemove)
	resultList = sql.getObjectsBy("MyBar", "", "")
	Assert(resultList.size() = 4)
	
	sql.deleteObjectsBy("MyBar", "id", "4")
	resultList = sql.getObjectsBy("MyBar", "", "")
	Assert(resultList.size() = 3)
	
	resultList = sql.getObjectsBy("MyFoo", "", "")
	Assert(resultList.size() = 7)
End Sub

Sub use_existing_db()
	Dim As PERSISTENCE_CLASS sql = Type<PERSISTENCE_CLASS>(DB_NAME, DB_HOST, DB_USER, DB_PWD)
	
	'Do some tests
	Dim As mdList(mdMapStringString) resultList
	
	resultList = sql.getObjectsBy("MyFoo", "", "")
	Assert(resultList.size() = 7)
End Sub

Sub remove_db()
	Dim As PERSISTENCE_CLASS sql = Type<PERSISTENCE_CLASS>(DB_NAME, DB_HOST, DB_USER, DB_PWD)
	
	'Delete the content of one table
	Dim As mdList(mdMapStringString) resultList
	
	resultList = sql.getObjectsBy("MyFoo", "", "")
	Assert(resultList.size() = 7)
	
	sql.deleteAllObjects("MyFoo")
	
	resultList = sql.getObjectsBy("MyFoo", "", "")
	Assert(resultList.size() = 0)
	
	'Remove the database
	sql.removeDatabase(DB_NAME)
End Sub

create_and_use_new_db()
use_existing_db()
remove_db()
